@@ -1,11 +1,11 @@
 /* 
  * The modifications to support SSLeay were done by Tim Hudson
- * tjh@mincom.oz.au
+ * tjh@cryptsoft.com
  *
  * You can do whatever you like with these patches except pretend that
  * you wrote them. 
  *
- * Email ssl-users-request@mincom.oz.au to get instructions on how to
+ * Email ssl-users-request@lists.cryptsoft.com to get instructions on how to
  * join the mailing list that discusses SSLeay and also these patches.
  *
  */
@@ -115,6 +115,7 @@ static char sccsid[] = "@(#)ftp.c	5.38 (Berkeley) 4/22/91";
 struct	sockaddr_in hisctladdr;
 struct	sockaddr_in data_addr;
 int	data = -1;
+int	pdata = -1;
 int	abrtflag = 0;
 int	ptflag = 0;
 struct	sockaddr_in myctladdr;
@@ -1172,6 +1173,38 @@ initconn()
 #ifdef USE_SOCKS
 	static int scanport = 5000;
 #endif /* USE_SOCKS */
+        int val[6];
+
+#ifdef PASVHACK
+       if( pasv_hack ) {
+         data_addr = myctladdr;
+         result = command("PASV");
+         a = (char *)&data_addr.sin_addr;
+         p = (char *)&data_addr.sin_port;
+
+         result = sscanf ( pasv, "%d,%d,%d,%d,%d,%d",
+	                        &val[0], &val[1], &val[2],
+                                &val[3], &val[4], &val[5]);
+
+         a[0] = val[0];
+         a[1] = val[1];
+         a[2] = val[2];
+         a[3] = val[3];
+         p[0] = val[4];
+         p[1] = val[5];
+
+         data = socket(AF_INET, SOCK_STREAM, 0);
+         if (data < 0) {
+           perror("ftp: data socket");
+           code = -1;
+           return (0);
+         }
+         result = CONNECT(data, (struct sockaddr *)&data_addr, sizeof (data_addr
+));
+         return ( result );
+       }
+#endif
+
 
 noport:
 	data_addr = myctladdr;
@@ -1251,22 +1284,28 @@ dataconn(lmode)
 {
 	struct sockaddr_in from;
 	int s, fromlen = sizeof (from), tos;
+	int ret;
 
-	s = ACCEPT(data, (struct sockaddr *) &from, &fromlen);
-	if (s < 0) {
-		perror("ftp: accept");
-		(void) close(data), data = -1;
-		return (NULL);
-	}
-	(void) close(data);
-	data = s;
+#ifdef PASVHACK
+        if ( !pasv_hack ) 
+#endif
+        {
+	    s = ACCEPT(data, (struct sockaddr *) &from, &fromlen);
+	    if (s < 0) {
+		    perror("ftp: accept");
+		    (void) close(data), data = -1;
+		    return (NULL);
+	    }
+	    (void) close(data);
+	    data = s;
 #ifdef IP_TOS
 #ifdef IPTOS_THROUGHPUT
 	tos = IPTOS_THROUGHPUT;
-	if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
+	if (setsockopt(data, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
 		perror("ftp: setsockopt TOS (ignored)");
 #endif
 #endif
+	}
 
 #ifdef USE_SSL
 	ssl_data_active_flag=0;
@@ -1279,6 +1318,8 @@ dataconn(lmode)
 	    ssl_data_con=(SSL *)SSL_new(ssl_ctx);
 
 	    SSL_set_fd(ssl_data_con,data);
+	    set_ssl_trace(ssl_data_con);
+
 	    SSL_set_verify(ssl_data_con,ssl_verify_flag,NULL);
 	    SSL_set_pref_cipher(ssl_data_con,getenv("SSL_CIPHER"));
 
@@ -1298,11 +1339,11 @@ dataconn(lmode)
 		fflush(stderr);
 	    }
 
-	    if (SSL_connect(ssl_data_con)<=0) {
+	    if ((ret=SSL_connect(ssl_data_con))<=0) {
                 static char errbuf[1024];
 
-	        sprintf(errbuf,"ftp: SSL_connect DATA error %s\n",
-			    ERR_error_string(ERR_get_error(),NULL));
+	        sprintf(errbuf,"ftp: SSL_connect DATA error %d - %s\n",
+			    ret,ERR_error_string(ERR_get_error(),NULL));
 
 		/* tell the user ... who else */
 		fprintf(stderr,"%s", errbuf);
@@ -1321,7 +1362,7 @@ dataconn(lmode)
 	    }
 
 	    if (debug) {
-		fprintf(stderr,"===>DONE SSL connect on DATA\n");
+		fprintf(stderr,"===>DONE SSL connect on DATA %d\n",data);
 		fflush(stderr);
 	    }
 
@@ -1739,6 +1780,8 @@ char *u,*p;
 		ssl_con=(SSL *)SSL_new(ssl_ctx);
 
 		SSL_set_fd(ssl_con,fileno(cout));
+		set_ssl_trace(ssl_con);
+
 		SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
 		SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
 
@@ -1747,7 +1790,7 @@ char *u,*p;
 		    if (!SSL_use_certificate_file(ssl_con, my_ssl_cert_file,
 			X509_FILETYPE_PEM)) {
 			fprintf(stderr,"%s: ",my_ssl_cert_file);
-			ERR_print_errors(stderr);
+			ERR_print_errors_fp(stderr);
 			exit(1);
 		    } else {
 			if (!my_ssl_key_file)
@@ -1755,7 +1798,7 @@ char *u,*p;
 			if (!SSL_use_RSAPrivateKey_file(ssl_con, my_ssl_key_file,
 			    X509_FILETYPE_PEM)) {
 			    fprintf(stderr,"%s: ", my_ssl_key_file);
-			    ERR_print_errors(stderr);
+			    ERR_print_errors_fp(stderr);
 			    exit(1);
 			}
 		    }
@@ -1768,7 +1811,7 @@ char *u,*p;
 				ERR_error_string(ERR_get_error(),NULL));
 		    perror(errbuf);
 		    /* abort time methinks ... */
-		    return ERROR;
+		    exit(1);
 		} else {
 		    fprintf(stderr,"[SSL Cipher %s]\n",SSL_get_cipher(ssl_con));
 		    fflush(stderr);
@@ -1833,10 +1876,16 @@ int
 ssl_getc(SSL *ssl_con)
 {
     char onebyte;
+    int ret;
 
-    if (SSL_read(ssl_con,&onebyte,1)!=1)
+    if ((ret=SSL_read(ssl_con,&onebyte,1))!=1) {
+      /* we want to know this stuff! */
+      if (ssl_debug_flag || (ret!=0)) {
+	  fprintf(stderr,"ssl_getc: SSL_read failed %d = %d\n",ret,errno);
+	  fflush(stderr);
+      }
       return -1;
-    else {
+    } else {
 	if (ssl_debug_flag && (SSL_LOG!=NULL)) {
 	    fprintf(SSL_LOG,"ssl_getc: SSL_read %d (%c) ",onebyte & 0xff,isprint(onebyte)?onebyte:'.');
 	    fflush(SSL_LOG);
@@ -1882,24 +1931,5 @@ ssl_putc(SSL *ssl_con,int oneint)
     return onebyte;
 }
 
-/* we really shouldn't have code like this! --tjh */
-int 
-old_ssl_putc(SSL *ssl_con,int oneint)
-{
-    char onebyte;
-
-    onebyte = oneint & 0xff;
-
-    if (SSL_write(ssl_con,&onebyte,1)!=1)
-      return -1;
-    else {
-	if (ssl_debug_flag && (SSL_LOG!=NULL)) {
-	    fprintf(SSL_LOG,"ssl_putc: SSL_write %d (%c) ",onebyte & 0xff,isprint(onebyte)?onebyte:'.');
-	    fflush(SSL_LOG);
-	}
-	return onebyte & 0xff;
-    }
-}
-
 #endif /* USE_SSL */
 
