@@ -77,35 +77,112 @@ char	*getlogin();
 
 #ifdef USE_SSL
 
+/* icky way of doing things ... */
+#include "../lib/sslapp.c"
+
 /*
 #include "ssl_err.h"
 */
 
 SSL *ssl_con;
-SSL_CTX *ssl_ctx;
 SSL *ssl_data_con;
-int ssl_debug_flag=0;
-int ssl_only_flag=0;
-int ssl_active_flag=0;
 int ssl_data_active_flag=0;
-int ssl_secure_flag=0;
-int ssl_certs_ok=0;
-int ssl_verify_flag=SSL_VERIFY_NONE;
-char ssl_file_path[1024];    /* don't look at that nasty value to the left */
 
 /* for the moment this is a compile time option only --tjh */
 int ssl_encrypt_data=1;
 int ssl_enabled=1;
 
+#if 0
+int ssl_active_flag=0;
+SSL_CTX *ssl_ctx;
+int ssl_debug_flag=0;
+int ssl_only_flag=0;
+int ssl_secure_flag=0;
+int ssl_certsok_flag=0;
+int ssl_verbose_flag=0;
+int ssl_verify_flag=SSL_VERIFY_NONE;
+char ssl_file_path[1024];    /* don't look at that nasty value to the left */
+#endif
+
 char *my_ssl_key_file=NULL;
 char *my_ssl_cert_file=NULL;
 
+BIO *bio_err=NULL;
+
+#if 0
+static void client_info_callback(s,where,ret)
+SSL *s;
+int where;
+int ret;
+{
+        if (where == SSL_CB_CONNECT_LOOP) {
+                fprintf(stderr,"SSL_connect:%s %s\n",SSL_state_string(s),SSL_state_string_long(s));
+        } else if (where == SSL_CB_CONNECT_EXIT) {
+                if (ret == 0) {
+                        fprintf(stderr,"SSL_connect:failed in %s %s\n",
+                                SSL_state_string(s),SSL_state_string_long(s));
+                } else if (ret < 0) {
+                        fprintf(stderr,"SSL_connect:error in %s %s\n",
+                                SSL_state_string(s),SSL_state_string_long(s));
+                }
+        }
+}
+#endif
+
+long bio_dump_cb(bio,cmd,argp,argi,argl,ret)
+BIO *bio;
+int cmd;
+char *argp;
+int argi;
+long argl;
+long ret;
+        {
+        BIO *out;
+
+/*
+        out=(BIO *)BIO_get_callback_arg(bio);
+*/
+	out=bio_err;
+        if (out == NULL) return(ret);
+
+        if (cmd == (BIO_CB_READ|BIO_CB_RETURN))
+                {
+                BIO_printf(out,"read from %08X (%d bytes => %ld (%X))\n",
+                        bio,argi,ret,ret);
+                BIO_dump(out,argp,(int)ret);
+		BIO_flush(out);
+                }
+        else if (cmd == (BIO_CB_WRITE|BIO_CB_RETURN))
+                {
+                BIO_printf(out,"write to %08X (%d bytes => %ld (%X))\n",
+                        bio,argi,ret,ret);
+                BIO_dump(out,argp,(int)ret);
+		BIO_flush(out);
+                }
+        return( (cmd & BIO_CB_RETURN) ? ret : 1);
+        }
+
+int set_ssl_trace(SSL *con) 
+{
+    if (con!=NULL) {
+	if (ssl_debug_flag) {
+	    BIO_set_callback(SSL_get_rbio(con),bio_dump_cb);
+	    BIO_set_callback_arg(SSL_get_rbio(con),bio_err);
+	}
+    }
+    return 0;
+}
+
 #endif /* USE_SSL */
 
 #ifdef USE_SOCKS
 int socks_flag=1;        /* socks usage is now the default */
 #endif /* USE_SOCKS */
 
+#ifdef PASVHACK
+int pasv_hack=1;
+#endif /* PASVHACK */
+
 main(argc, argv)
 	char *argv[];
 {
@@ -168,7 +245,10 @@ main(argc, argv)
 				    ssl_secure_flag=1;
 				}
 				if (strcmp(optarg, "certsok") == 0 ) {
-				    ssl_certs_ok=1;
+				    ssl_certsok_flag=1;
+				}
+				if (strcmp(optarg, "verbose") == 0 ) {
+				    ssl_verbose_flag=1;
 				}
 				if (strncmp(optarg, "verify=", strlen("verify=")) == 0 ) {
 				    ssl_verify_flag=atoi(optarg+strlen("verify="));
@@ -196,6 +276,16 @@ main(argc, argv)
 				    fflush(stderr);
 #endif /* USE_SOCKS */
 				}
+                                if ( (strcmp(optarg, "!pasv") == 0 ) ||
+                                     (strcmp(optarg, "nopasv") == 0 ) ) {
+#ifdef PASVHACK
+                                    pasv_hack=0;
+#else /* !PASVHACK */
+                                    fprintf(stderr,"No PASVHACK support\n");
+				    fflush(stderr);
+#endif /* PASVHACK */
+                                }
+
                                 /* we have swallowed an extra arg */
 				argc--;
                                 argv++;
@@ -277,34 +367,12 @@ main(argc, argv)
 
 #ifdef USE_SSL
         if (ssl_enabled) {
-	    int i;
-
-	    if (ssl_debug_flag) {
-		fprintf(stderr,"SSL DEBUG ACTIVE\n");
+	    if (!do_ssleay_init(0)) {
+		fprintf(stderr,"ftp: SSLeay initialisation failed\n");
 		fflush(stderr);
-                /* for the moment I want the output on screen */
-		if (debug) {
-		    SSL_ERR=SSL_LOG=stderr;
-		} else {
-		    (void)freopen("ftp.err","w",stderr);
-		    SSL_ERR=stderr;
-		    SSL_debug("ftp.log");
-		}
-	    } else {
-		/* disable all the debug and trace */
-		SSL_LOG=SSL_ERR=NULL;
-	    }
-	    SSL_load_error_strings();
-
-	    ssl_ctx=(SSL_CTX *)SSL_CTX_new();
-
-            if (!X509_set_default_verify_paths(ssl_ctx->cert)) {
-		fprintf(stderr,"ftp: cannot load certificates via X509_set_default_verify_paths\n");
-		fflush(stderr);
-		ERR_print_errors(stderr);
+		ERR_print_errors_fp(stderr);
 		exit(1);
 	    }
-
 	}
 #endif /* USE_SSL */
 
